/**
 * Copyright 2025 cronn GmbH
 * SPDX-License-Identifier: AGPL-3.0-only
 */

import { ChildExamination } from "@eshg/dental/api/models/ChildExamination";
import { useDeleteProphylaxisSessionParticipantOptions } from "@eshg/dental/api/mutations/prophylaxisSessionApi";
import { routes } from "@eshg/dental/shared/routes";
import { GENDER_VALUES } from "@eshg/lib-portal/components/formFields/constants";
import { InternalLinkButton } from "@eshg/lib-portal/components/navigation/InternalLinkButton";
import { formatDate } from "@eshg/lib-portal/formatters/dateTime";
import { Add } from "@mui/icons-material";
import CancelIcon from "@mui/icons-material/Cancel";
import DeleteIcon from "@mui/icons-material/DeleteOutlined";
import { Button, Divider, Stack, Typography } from "@mui/joy";
import { createColumnHelper } from "@tanstack/react-table";
import { useEffect, useState } from "react";
import { isDefined } from "remeda";

import { ExaminationStatusChip } from "@/lib/businessModules/dental/features/examinations/ExaminationStatusChip";
import { useAddChildToProphylaxisSessionSidebar } from "@/lib/businessModules/dental/features/prophylaxisSessions/AddChildToProphylaxisSessionSidebar";
import { ChangeReasonForAbsenceModal } from "@/lib/businessModules/dental/features/prophylaxisSessions/ChangeReasonForAbsenceModal";
import {
  useFilteredParticipants,
  useProphylaxisSessionStore,
} from "@/lib/businessModules/dental/features/prophylaxisSessions/prophylaxisSessionStore/ProphylaxisSessionStoreProvider";
import {
  FluoridationConsentFilter,
  GenderFilter,
} from "@/lib/businessModules/dental/features/prophylaxisSessions/prophylaxisSessionStore/participantFilters";
import {
  ParticipantSortKey,
  ParticipantSorting,
} from "@/lib/businessModules/dental/features/prophylaxisSessions/prophylaxisSessionStore/participantSorting";
import { OverlayBoundary } from "@/lib/shared/components/boundaries/OverlayBoundary";
import { ActionsMenu } from "@/lib/shared/components/buttons/ActionsMenu";
import { ButtonBar } from "@/lib/shared/components/buttons/ButtonBar";
import { DataTable } from "@/lib/shared/components/table/DataTable";
import { TablePage } from "@/lib/shared/components/table/TablePage";
import { displayBoolean } from "@/lib/shared/helpers/booleans";
import {
  CustomSortingProps,
  useTableControl,
} from "@/lib/shared/hooks/searchParams/useTableControl";
import { useConfirmationDialog } from "@/lib/shared/hooks/useConfirmationDialog";

import { ParticipantFilter, ParticipantFilterDef } from "./ParticipantFilter";

const GENDER_FILTERS: ParticipantFilterDef<GenderFilter>[] = [
  { label: "Alle", value: "ANY" },
  { label: "männlich", value: "MALE" },
  { label: "weiblich", value: "FEMALE" },
];

const FLUORIDATION_CONSENT_FILTERS: ParticipantFilterDef<FluoridationConsentFilter>[] =
  [
    { label: "Alle", value: "ANY" },
    { label: "Ja", value: "YES" },
    { label: "Nein", value: "NO" },
  ];

export function ProphylaxisSessionParticipantsTable() {
  const prophylaxisSessionId = useProphylaxisSessionStore((state) => state.id);
  const isScreening = useProphylaxisSessionStore((state) => state.isScreening);
  const isFluoridation = isDefined(
    useProphylaxisSessionStore((state) => state.fluoridationVarnish),
  );
  const isExamination = isFluoridation || isScreening;
  const prophylaxisSessionVersion = useProphylaxisSessionStore(
    (state) => state.version,
  );
  const allParticipants = useProphylaxisSessionStore(
    (state) => state.participants,
  );
  const filteredParticipants = useFilteredParticipants();
  const participantSorting = useProphylaxisSessionStore(
    (state) => state.participantSorting,
  );
  const setParticipantSorting = useProphylaxisSessionStore(
    (state) => state.setParticipantSorting,
  );
  const { openConfirmationDialog } = useConfirmationDialog();

  const deleteOptions = useDeleteProphylaxisSessionParticipantOptions(
    prophylaxisSessionId,
    prophylaxisSessionVersion,
    allParticipants,
  );
  const [
    openReasonForAbsenceModalOfParticipant,
    setOpenReasonForAbsenceModalOfParticipant,
  ] = useState<ChildExamination>();

  function handleRemoveParticipant(childExternalId: string) {
    openConfirmationDialog({
      color: "danger",
      title: "Kind entfernen?",
      confirmLabel: "Entfernen",
      description: "Möchten Sie das Kind aus der Prophylaxe entfernen?",
      onConfirmMutation: {
        mutationOptions: deleteOptions,
        variableSupplier: () => childExternalId,
      },
      onConfirm: () => Promise.resolve(),
    });
  }

  function handleAbsentParticipant(examination: ChildExamination) {
    setOpenReasonForAbsenceModalOfParticipant(examination);
  }

  function closeAbsenceModal() {
    setOpenReasonForAbsenceModalOfParticipant(undefined);
  }

  function routeToExamination(participantIndex: number) {
    return routes.prophylaxisSessions
      .byId(prophylaxisSessionId)
      .examinations.byIndex(participantIndex);
  }

  const tableControl = useTableControl({
    serverSideSorting: true,
    sortFieldName: "sortKey",
    sortDirectionName: "sortDirection",
    initialSorting: {
      id: participantSorting.sortKey,
      desc: participantSorting.sortDirection === "desc",
    },
  });

  const { sortKey, sortDirection } = resolveTableSorting(
    tableControl.tableSorting,
  );
  useEffect(() => {
    setParticipantSorting({
      sortKey,
      sortDirection,
    });
  }, [setParticipantSorting, sortKey, sortDirection]);

  return (
    <TablePage
      controls={
        <ButtonBar
          left={
            <>
              <Stack>
                <Typography level="h4" component="h2" marginBottom={1}>
                  Teilnehmende Kinder
                </Typography>
                <Stack
                  direction="row"
                  gap={3}
                  alignItems="center"
                  flexWrap="wrap"
                >
                  <ParticipantFilter
                    name="gender"
                    label="Geschlecht"
                    filters={GENDER_FILTERS}
                  />
                  {isFluoridation && (
                    <>
                      <Divider orientation="vertical" />
                      <ParticipantFilter
                        name="fluoridationConsentGiven"
                        label="Fluoridierungseinverständnis"
                        filters={FLUORIDATION_CONSENT_FILTERS}
                      />
                    </>
                  )}
                </Stack>
              </Stack>
            </>
          }
          right={
            <>
              <AddChildButton />
              {isExamination && filteredParticipants.length > 0 && (
                <InternalLinkButton href={routeToExamination(0)}>
                  Prophylaxe starten
                </InternalLinkButton>
              )}
            </>
          }
        />
      }
    >
      <DataTable
        data={filteredParticipants}
        columns={columnDefs(
          handleRemoveParticipant,
          handleAbsentParticipant,
          isFluoridation,
          isExamination,
        )}
        rowNavigation={
          isExamination
            ? {
                focusColumnAccessorKey: "lastName",
                route: (row) => routeToExamination(row.index),
              }
            : undefined
        }
        sorting={tableControl.tableSorting}
        enableSortingRemoval={false}
        minWidth={1200}
      />
      {isDefined(openReasonForAbsenceModalOfParticipant) && (
        <OverlayBoundary>
          <ChangeReasonForAbsenceModal
            onClose={closeAbsenceModal}
            examination={openReasonForAbsenceModalOfParticipant}
          />
        </OverlayBoundary>
      )}
    </TablePage>
  );
}

const columnHelper = createColumnHelper<ChildExamination>();

function columnDefs(
  onRemoveParticipant: (participantId: string) => void,
  onAbsentParticipant: (examination: ChildExamination) => void,
  isFluoridation: boolean,
  isExamination: boolean,
) {
  return [
    columnHelper.accessor("firstName", {
      header: "Vorname",
      cell: (props) => props.getValue(),
      enableSorting: true,
      meta: {
        canNavigate: { parentRow: true },
        width: 120,
      },
    }),
    columnHelper.accessor("lastName", {
      header: "Nachname",
      cell: (props) => props.getValue(),
      enableSorting: true,
      meta: {
        canNavigate: { parentRow: true },
        width: 120,
      },
    }),
    columnHelper.accessor("dateOfBirth", {
      header: "Geburtsdatum",
      cell: (props) => formatDate(props.getValue()),
      enableSorting: true,
      meta: {
        canNavigate: { parentRow: true },
        width: 120,
      },
    }),
    columnHelper.accessor("groupName", {
      header: "Gruppe",
      cell: (props) => props.getValue(),
      enableSorting: true,
      meta: {
        canNavigate: { parentRow: true },
        width: 90,
      },
    }),
    columnHelper.accessor("gender", {
      header: "Geschlecht",
      cell: (props) => {
        const value = props.getValue();
        return isDefined(value) ? GENDER_VALUES[value] : "";
      },
      enableSorting: true,
      meta: {
        canNavigate: { parentRow: true },
        width: 110,
      },
    }),
    ...(isFluoridation
      ? [
          columnHelper.accessor("currentFluoridationConsent", {
            header: "Fluoridierungseinverständnis",
            cell: (props) => displayBoolean(props.getValue()?.consented),
            enableSorting: true,
            meta: {
              canNavigate: { parentRow: true },
              width: 205,
            },
          }),
        ]
      : []),
    ...(isExamination
      ? [
          columnHelper.accessor("status", {
            header: "Status",
            cell: (props) => (
              <ExaminationStatusChip status={props.getValue()} />
            ),
            enableSorting: true,
            meta: {
              canNavigate: { parentRow: true },
              width: 110,
            },
          }),
        ]
      : []),
    columnHelper.display({
      header: "Aktionen",
      id: "actions",
      cell: (props) =>
        childCanBeRemoved(props.row.original) ? (
          <ActionsMenu
            actionItems={[
              ...(props.row.original.status !== "CLOSED" && isExamination
                ? [
                    {
                      label: "Nicht anwesend",
                      startDecorator: <CancelIcon />,
                      onClick: () => onAbsentParticipant(props.row.original),
                    },
                  ]
                : []),
              {
                label: "Entfernen",
                startDecorator: <DeleteIcon />,
                onClick: () => onRemoveParticipant(props.row.original.childId),
                color: "danger",
              },
            ]}
          />
        ) : undefined,
      meta: {
        width: 80,
        cellStyle: "button",
        textAlign: "right",
      },
    }),
  ];
}

function resolveTableSorting(
  tableSorting: CustomSortingProps,
): ParticipantSorting {
  if (!tableSorting.manualSorting) {
    throw new Error("Table does not use server side sorting");
  }

  const sortingState = tableSorting.sortingState.at(0);
  if (sortingState === undefined) {
    throw new Error("Table does not have a sorting state");
  }

  return {
    sortKey: sortingState.id as ParticipantSortKey,
    sortDirection: sortingState.desc ? "desc" : "asc",
  };
}

function AddChildButton() {
  const prophylaxisSessionId = useProphylaxisSessionStore((state) => state.id);
  const prophylaxisSessionVersion = useProphylaxisSessionStore(
    (state) => state.version,
  );
  const institutionId = useProphylaxisSessionStore(
    (state) => state.institution.id,
  );
  const allParticipants = useProphylaxisSessionStore(
    (state) => state.participants,
  );
  const sidebar = useAddChildToProphylaxisSessionSidebar();

  return (
    <Button
      startDecorator={<Add />}
      onClick={() =>
        sidebar.open({
          prophylaxisSessionId,
          prophylaxisSessionVersion,
          institutionId,
          allParticipants,
        })
      }
    >
      Kind hinzufügen
    </Button>
  );
}

function childCanBeRemoved(child: ChildExamination) {
  return child.result === undefined;
}
